/* Copyright (c) 1999, 2000 TTK Ciar / Bill Moyer
 * All rights reserved.                               
 *
 * This code is derived from code developed by TTK Ciar and Bill Moyer.    
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions         
 * are met:                                                                  
 *
 *  1. Redistributions of source code must retain the above copyright        
 *     notice, this list of conditions and the following disclaimer.
 *  2. The name of the author may not be used to endorse or promote
 *     products derived from this software without specific prior
 *     written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
// TTK here -- Clarification on above: This license is NOT viral and
// does NOT put you under any obligation to do ANY of the following:
//   ** distribute sources
//   ** contribute modifications
//   ** grant credit or show recognition
//   ** grant any other kind of compensation
// This document may be used and modified for free OR COMMERCIAL 
// development with impunity.  The SOLE intention of this license is 
// to prevent anyone from laying private claim to publicly available 
// versions of the source code and putting any kind of restrictions 
// on its distribution or use.

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "qd3lib.h"

static int major_revision = 3;
static int minor_revision = 1;

static int   width      = 16;      /* number of bytes processed per line */
static int   hexcase    =  0;    /* 0 for lowercase hex, 1 for uppercase */
static int   useheader  =  1;  /* 1 to generate filename/filesize prefix line */

static int   pix = 0;
static char  paddedstr[8][128];

/* interface */ int  qd3ctl ( int what, int val ) {
  switch ( what ) {
    case QD_GETWIDTH : return width;
    case QD_SETWIDTH : width = val; return 0;
    case QD_GETCASE  : return hexcase;
    case QD_SETCASE  : hexcase = val; return 0;
    case QD_GETHEAD  : return useheader;
    case QD_SETHEAD  : useheader = val; return 0;
    }
  return -1;
  }

/* interface */ char *padstr ( char *str, char pad, int w_ ) {
  int len, r, i, w;
  if ( pix >= 8 ) pix = 0;
  paddedstr[pix][0] = 0;
  if ( str == NULL ) return paddedstr[pix++];
  w = w_;
  if ( w > 127 ) return paddedstr[pix++];
  len = strlen(str);
  if ( len >= w ) { 
    strcpy ( paddedstr[pix], str ); 
    return ( paddedstr[pix++] );
    }
  r = w - len;
  for ( i = 0; i < r; i++ ) paddedstr[pix][i] = pad;
  strcpy ( &(paddedstr[pix][r]), str );
  return ( paddedstr[pix++] );
  }

/* interface */ char *padnum ( int num, int w ) {
  char numstr[32];
  sprintf ( numstr, "%d", num );
  return padstr ( numstr, '0', 2 );
  }

/* interface */ char *padhex ( int num, int w, int ucase ) {
  char numstr[32];
  if ( ucase ) 
       sprintf ( numstr, "%X", num );
  else sprintf ( numstr, "%x", num );
  return padstr ( numstr, '0', 2 );
  }

/* interface */ int qd3 ( char *name, int k, char *buf ) {
   int  n, i, j, bit, quit, res;
   char *asciistr;
   int  idatasectionindex = 0;
   int  fsz;
   unsigned char foo;

   asciistr = (char*) malloc ( width + 1 );
   asciistr[width] = 0;
   if ( useheader ) printf ("[%s] %d\n",name,k);
   n = 0; i = 0; j = 0; quit = 0;
   while ( !quit ) {
     if ( n < k ) foo = buf[n++];
     else {
       res = 0;
       foo = 0;
       if ( i % width == 0 ) quit = 1;
       }
     if ( i % width == 0 ) if ( !quit )
       printf ( "%6u %5X : ", j, j );
     asciistr[i] = foo;
     if ( ( (unsigned char)asciistr[i] < ' ' ) ||
          ( (unsigned char)asciistr[i] > 126 )    )
        asciistr[i] = '.';
     if ( !quit || i ) {
        if ( res == 0 ) printf (" **");
        else printf ("%3s",padhex(foo,2,hexcase));
        }
     i++; j++;
     if ( i % width == 0 ) { 
       asciistr[i] = 0;
       i = 0; 
       printf (" |%s\n",asciistr); 
       }
     }
   free ( asciistr );
   return 0;
   }

/* interface */ int qd3_to_buf ( char *name, int k, char *buf, char *dest, int *max_ ) {
   int  n, i, j, bit, quit, res;
   char *asciistr;
   int  idatasectionindex = 0;
   int  fsz;
   int  max = *max_;
   char scratch[1024];
   unsigned char foo;
   int  ix, ist, kst;

   asciistr = (char*) malloc ( width + 1 );
   asciistr[width] = 0;
   ix = 0;
   if ( useheader ) 
     { 
       sprintf (scratch, "[%s] %d\n", name, k );
       for ( ist = 0, kst = strlen(scratch); ( ist < kst ) && ( ix < max ); ist++, ix++ ) dest[ix] = scratch[ist];
     }
   n = 0; i = 0; j = 0; quit = 0;
   while ( !quit ) {
     if ( n < k ) foo = buf[n++];
     else {
       res = 0;
       foo = 0;
       if ( i % width == 0 ) quit = 1;
       }

     if ( i % width == 0 ) if ( !quit )
       { 
         sprintf ( scratch, "%6u %5X : ", j, j );
         for ( ist = 0, kst = strlen(scratch); ( ist < kst ) && ( ix < max ); ist++, ix++ ) dest[ix] = scratch[ist];
       }
       
     asciistr[i] = foo;
     if ( ( (unsigned char)asciistr[i] < ' ' ) ||
          ( (unsigned char)asciistr[i] > 126 )    )
        asciistr[i] = '.';
     if ( !quit || i ) {
        if ( res == 0 ) 
          { 
            sprintf (scratch, " **" );
            for ( ist = 0, kst = strlen(scratch); ( ist < kst ) && ( ix < max ); ist++, ix++ ) dest[ix] = scratch[ist];
          }
        else 
          { 
            sprintf (scratch, "%3s",padhex(foo,2,hexcase));
            for ( ist = 0, kst = strlen(scratch); ( ist < kst ) && ( ix < max ); ist++, ix++ ) dest[ix] = scratch[ist];
          }
        }
     i++; j++;
     if ( i % width == 0 ) { 
       asciistr[i] = 0;
       i = 0; 
         { 
           sprintf ( scratch, " |%s\n",asciistr); 
           for ( ist = 0, kst = strlen(scratch); ( ist < kst ) && ( ix < max ); ist++, ix++ ) dest[ix] = scratch[ist];
         }
       }
     }
   free ( asciistr );
   // deliberately overwriting trailing \n
   dest[ix-1] = 0;
   *max_ = ix;
   return 0;
   }
